'use strict';

const fs = require('fs');
const kit = require('@nuxt/kit');
const upath = require('upath');
const globby = require('globby');
const scule = require('scule');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e['default'] : e; }

const fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
const globby__default = /*#__PURE__*/_interopDefaultLegacy(globby);

function sortDirsByPathLength({ path: pathA }, { path: pathB }) {
  return pathB.split(/[\\/]/).filter(Boolean).length - pathA.split(/[\\/]/).filter(Boolean).length;
}
function hyphenate(str) {
  return str.replace(/\B([A-Z])/g, "-$1").toLowerCase();
}
async function scanComponents(dirs, srcDir) {
  const components = [];
  const filePaths = new Set();
  const scannedPaths = [];
  for (const { path, pattern, ignore = [], prefix, extendComponent, pathPrefix, level, prefetch = false, preload = false } of dirs.sort(sortDirsByPathLength)) {
    const resolvedNames = new Map();
    for (const _file of await globby__default(pattern, { cwd: path, ignore })) {
      const filePath = upath.join(path, _file);
      if (scannedPaths.find((d) => filePath.startsWith(d))) {
        continue;
      }
      if (filePaths.has(filePath)) {
        continue;
      }
      filePaths.add(filePath);
      const prefixParts = [].concat(prefix ? scule.splitByCase(prefix) : [], pathPrefix !== false ? scule.splitByCase(upath.relative(path, upath.dirname(filePath))) : []);
      let fileName = upath.basename(filePath, upath.extname(filePath));
      if (fileName.toLowerCase() === "index") {
        fileName = pathPrefix === false ? upath.basename(upath.dirname(filePath)) : "";
      }
      const fileNameParts = scule.splitByCase(fileName);
      const componentNameParts = [];
      while (prefixParts.length && (prefixParts[0] || "").toLowerCase() !== (fileNameParts[0] || "").toLowerCase()) {
        componentNameParts.push(prefixParts.shift());
      }
      const componentName = scule.pascalCase(componentNameParts) + scule.pascalCase(fileNameParts);
      if (resolvedNames.has(componentName)) {
        console.warn(`Two component files resolving to the same name \`${componentName}\`:

 - ${filePath}
 - ${resolvedNames.get(componentName)}`);
        continue;
      }
      resolvedNames.set(componentName, filePath);
      const pascalName = scule.pascalCase(componentName).replace(/["']/g, "");
      const kebabName = hyphenate(componentName);
      const shortPath = upath.relative(srcDir, filePath);
      const chunkName = "components/" + kebabName;
      let component = {
        filePath,
        pascalName,
        kebabName,
        chunkName,
        shortPath,
        export: "default",
        global: Boolean(global),
        level: Number(level),
        prefetch: Boolean(prefetch),
        preload: Boolean(preload)
      };
      if (typeof extendComponent === "function") {
        component = await extendComponent(component) || component;
      }
      const definedComponent = components.find((c) => c.pascalName === component.pascalName);
      if (definedComponent && component.level < definedComponent.level) {
        Object.assign(definedComponent, component);
      } else if (!definedComponent) {
        components.push(component);
      }
    }
    scannedPaths.push(path);
  }
  return components;
}

const isPureObjectOrString = (val) => !Array.isArray(val) && typeof val === "object" || typeof val === "string";
const isDirectory = (p) => {
  try {
    return fs__default.statSync(p).isDirectory();
  } catch (_e) {
    return false;
  }
};
const module$1 = kit.defineNuxtModule({
  name: "components",
  defaults: {
    dirs: ["~/components"]
  },
  setup(options, nuxt) {
    let componentDirs = [];
    nuxt.hook("app:resolve", async () => {
      await nuxt.callHook("components:dirs", options.dirs);
      componentDirs = options.dirs.filter(isPureObjectOrString).map((dir) => {
        const dirOptions = typeof dir === "object" ? dir : { path: dir };
        const dirPath = kit.resolveAlias(dirOptions.path, nuxt.options.alias);
        const transpile = typeof dirOptions.transpile === "boolean" ? dirOptions.transpile : "auto";
        const extensions = dirOptions.extensions || ["vue"];
        dirOptions.level = Number(dirOptions.level || 0);
        const present = isDirectory(dirPath);
        if (!present && dirOptions.path !== "~/components") {
          console.warn("Components directory not found: `" + dirPath + "`");
        }
        return {
          ...dirOptions,
          enabled: true,
          path: dirPath,
          extensions,
          pattern: dirOptions.pattern || `**/*.{${extensions.join(",")},}`,
          ignore: [
            "**/*.stories.{js,ts,jsx,tsx}",
            "**/*{M,.m,-m}ixin.{js,ts,jsx,tsx}",
            "**/*.d.ts",
            ...dirOptions.ignore || []
          ],
          transpile: transpile === "auto" ? dirPath.includes("node_modules") : transpile
        };
      }).filter((d) => d.enabled);
      nuxt.options.build.transpile.push(...componentDirs.filter((dir) => dir.transpile).map((dir) => dir.path));
    });
    nuxt.hook("app:templates", async (app) => {
      const components = await scanComponents(componentDirs, nuxt.options.srcDir);
      await nuxt.callHook("components:extend", components);
      if (!components.length) {
        return;
      }
      app.templates.push({
        path: "components.mjs",
        src: upath.resolve(__dirname, "runtime/components.tmpl.mjs"),
        data: { components }
      });
      app.templates.push({
        path: "components.d.ts",
        src: upath.resolve(__dirname, "runtime/components.tmpl.d.ts"),
        data: { components }
      });
      app.plugins.push({ src: "#build/components" });
    });
    nuxt.hook("builder:watch", async (event, path) => {
      if (!["add", "unlink"].includes(event)) {
        return;
      }
      const fPath = upath.resolve(nuxt.options.rootDir, path);
      if (componentDirs.find((dir) => fPath.startsWith(dir.path))) {
        await nuxt.callHook("builder:generateApp");
      }
    });
  }
});

module.exports = module$1;
